use Test::More tests => 1;

BEGIN {
use_ok( 'Maypole::Plugin::Config::Apache' );
}

diag( "Testing Maypole::Plugin::Config::Apache $Maypole::Plugin::Config::Apache::VERSION, Perl 5.008006, /usr/local/bin/perl" );
